
//***********************************************************************************
//FILE: genPreInstall.java
//PURPOSE: This program reads the OraDDL\Table.ddl and OraDDL\Index.dll and generate 
//         PreInstall.cfg.template
//***********************************************************************************

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;

public class genPreInstallCfg {

//################################################
//STATIC CLASS VARIABLES
//################################################
//CONTANT INTEGERS
public static final int			STRINGBUFFER_SIZE = 256;
public static final int 		OK=0;
public static final int 		ERROR=-1;
public static final char		UNIX='2';
public static final char		WINDOW='1';

//CONSTANT STRINGS
public static final String		IN_CRTAB_FILE	="Table.ddl";
public static final String		IN_CRIDX_FILE	="Index.ddl";

static char				slash;
static char				platform;
static char				dquote	;
static char				squote;
static int				nbtables=0;
static int				nbindexes=0;
static StringBuffer			outdir;
static StringBuffer 			outfile;
static StringBuffer			orahome_dir,data_dir,admin_dir;

static String				orahome,oradata,oraadmin;
static String				out_idxscript,out_tabscript,out_altertabscr,out_alteridxscr;
static String				in_tabmap,in_idxmap,in_tabscript,in_idxscript;
static boolean				debug_mode;
static Hashtable			hash4tab,hash4idx ;
static Date				today;
static String				saved_on;
static RandomAccessFile			fouttab;

//##############################################################################
//MAIN
//##############################################################################
public static void main(String argv[])
{
	int let=0;
	byte line[]=new byte[100];
	StringBuffer str,indir;
	int ret;
	int cnt_braces=-1;
	try
	{
		
		//CHECK COMMAND LINE OPTION
		if (argv.length ==1 && argv[0].compareToIgnoreCase("-v")==0)
		{
			System.out.println("Version 1 built on Sept 10, 2003.");
			return ;
		}
		//INIT FILE NAMES THEN CHECK IF FILES EXIST
		orahome_dir	= new StringBuffer("ora92").append(File.separatorChar);
		data_dir	= new StringBuffer("oradata").append(File.separatorChar);
		admin_dir	= new StringBuffer("admin").append(File.separatorChar);
		indir = new StringBuffer("..").append(File.separatorChar);
		indir.append("OraDDL").append(File.separatorChar);
		in_tabscript = new String(indir.toString()) + IN_CRTAB_FILE;
		in_idxscript = new String(indir.toString()) + IN_CRIDX_FILE;
		out_tabscript  	= new String("PreInstall.cfg.template");

		//PARSE CONFIG FILE & GENERATE OUTPUTFILES
		fouttab 	= new RandomAccessFile(out_tabscript,"rw");
		if (genHeader()==OK && processTableDDL() == OK && processIndexDDL()==OK)
		{
			System.out.println("Completed successfully. The 'PreInstall.cfg.template' is created.");
			fouttab.close();
		}
		else
		{
			System.out.println("Terminated unsuccessfully.");
		}
	}
	catch(Exception e)
	{
		System.out.println("Exception. Error Message: " + e);
		//return ERROR;
	}
     }	//end of main
//###################################################################
//genHeader()
//###################################################################
public static int genHeader()
{
try
{
	fouttab.writeBytes("#####################################################################\n");
	fouttab.writeBytes("#FILE :		PreInstall.cfg\n");
	fouttab.writeBytes("#PURPOSE: This file is used to generate the necessary scripts for the \n");
	fouttab.writeBytes("#         PMO installation\n");
	fouttab.writeBytes("#####################################################################\n");

	fouttab.writeBytes("\n[INSTANCE]\n");
	fouttab.writeBytes("db_name=PMOFFICE\n");
	fouttab.writeBytes("db_domain=corp.hq.systemcorp.com\n");
	fouttab.writeBytes("oracle_home ="+ orahome_dir.toString() + "\n");
	fouttab.writeBytes("admin_directory=" + admin_dir + "\n");       
	fouttab.writeBytes("datafile_directory ="  +  data_dir + "\n");

	fouttab.writeBytes("sys_password=change_on_install\n");
	fouttab.writeBytes("db_cache_size= 134217728\n");
	fouttab.writeBytes("shared_pool_size= 335544320\n");
	fouttab.writeBytes("java_pool_size= 64000000\n");
	fouttab.writeBytes("large_pool_size = 33554432\n");
	fouttab.writeBytes("sga_max_size = 1292969220\n");
	fouttab.writeBytes("shared_pool_reserved_size=32000000\n");
	fouttab.writeBytes("pga_aggregate_target=25165824\n");
	fouttab.writeBytes("option_inter_media=yes\n");
	fouttab.writeBytes("option_oracle_text=yes\n");

	fouttab.writeBytes("\n[TABLESPACE]\n");
	fouttab.writeBytes("tablespace=PMO_DATA_64K\n");
	fouttab.writeBytes("extent_size=64K\n");
	fouttab.writeBytes("datafile="  +  data_dir + "PMO_DATA_64K001.dbf\n");

	fouttab.writeBytes("\n[TABLESPACE]\n");
	fouttab.writeBytes("tablespace=PMO_LOB_64K\n");
	fouttab.writeBytes("extent_size=64K\n");
	fouttab.writeBytes("datafile="  +  data_dir + "PMO_LOB_64K001.dbf\n");

	fouttab.writeBytes("\n[TABLESPACE]\n");
	fouttab.writeBytes("tablespace=PMO_IDX_64K\n");
	fouttab.writeBytes("extent_size=64K\n");
	fouttab.writeBytes("datafile="  +  data_dir + "PMO_IDX_64K001.dbf\n");

	fouttab.writeBytes("\n[TABLESPACE]\n");
	fouttab.writeBytes("tablespace=PMO_DATA_64M\n");
	fouttab.writeBytes("extent_size=64M\n");
	fouttab.writeBytes("datafile="  +  data_dir + "PMO_DATA_64M001.dbf\n");

	fouttab.writeBytes("\n[TABLESPACE]\n");
	fouttab.writeBytes("tablespace=PMO_LOB_64M\n");
	fouttab.writeBytes("extent_size=64M\n");
	fouttab.writeBytes("datafile="  +  data_dir + "PMO_LOB_64M001.dbf\n");


	fouttab.writeBytes("\n[OWNER]\n");
	fouttab.writeBytes("owner=PMO\n");
	fouttab.writeBytes("password=pmo\n");
	fouttab.writeBytes("default_tablespace=PMO_DATA_64K\n");

	fouttab.writeBytes("\n[CONNUSER]\n");
	fouttab.writeBytes("connuser=PMO2\n");
	fouttab.writeBytes("password=pmo2\n");
	fouttab.writeBytes("default_tablespace=PMO_DATA_64K\n");
   	return OK;
}
catch(Exception e)
{
    System.out.println("Exception in genHeader(): " + e);
    return ERROR;
}
}

//##############################################################################
//processTableDDL:
//##############################################################################
public static int processTableDDL()
{
String line,line0;
StringBuffer linebuf,tablename;
StringBuffer lobcolumns;
RandomAccessFile fintab =null;
boolean searchtab,tbs_already_specified=false;
int inspos,pos,seppos,lobpos,linenum=0 ;
char let;
int cnt_braces=-1;
try
{
	fintab 		= new RandomAccessFile(in_tabscript,"r");
	searchtab	= false;
	tablename = new StringBuffer(STRINGBUFFER_SIZE);
	lobcolumns = new StringBuffer(STRINGBUFFER_SIZE);
	linebuf= new StringBuffer(STRINGBUFFER_SIZE); 

	while(true)
	{
	    if ((line0 = fintab.readLine()) ==null)
		break;
	    linenum++;
	    
	    //FORMAT THE LINE
	    line = line0.trim().toUpperCase();
	    if (line.length() < 1)
	    {
	    	fouttab.writeBytes(line0+"\n");
	    	continue;
	    }
	    
	    //SEARCH FOR "CREATE TABLE" AND TABLE NAME
	    if (line.indexOf("CREATE")!= -1 && (pos=line.indexOf("TABLE")) != -1)
	    {
	    	//RESET EVERYTHING
	    	searchtab=true;
	    	pos = pos + 6;
	    }
	    else pos = 0;
	    
	    if (searchtab==true)
	    {
	        for (int j=pos; j< line.length(); j++)
		{
		    	let = line.charAt(j);
		    	//System.out.println("LET = " + let );
		    	if ((let==' ' || let=='\t' || let=='(') && tablename.length() > 1)		    	
		    		break;
		    	else
		    		tablename.append(let);
		}
		searchtab=false;
	    }
	    
	    //SEARCH FOR LOB COLUMNS
	    if (  (lobpos= line.indexOf(" BLOB"))!= -1 ||(lobpos= line.indexOf(" CLOB")) != -1
	        ||(lobpos= line.indexOf("\tBLOB"))!= -1 ||(lobpos= line.indexOf("\tCLOB")) != -1)
	    {
	    	//System.out.println("TRACE> LINENUM="+linenum + ";pos="+lobpos+ "; LOBCOL=" + line);
	    	if (lobcolumns.length() >0) lobcolumns.append(", ");	
	    	int startpos = -1;
	    	for (int j=lobpos-1; j>=0; j--)
		{
		    	let = line.charAt(j);
		    	if (let != ' ' && let != '\t' && let != '(')
		    		startpos = j;
		    	else 
		    	{
		    	    if (startpos != -1) break;
		    	}	
		}
		if (startpos != -1) lobcolumns.append(line.substring(startpos,lobpos));
	    }
	    
	    //WRITE TO FILE IF FOUND ;
	    if ((line.indexOf(";"))!= -1)
	    {
	    	//System.out.println("processTableDDL> TABLE=" + tablename +", LOB=" + lobcolumns);
		fouttab.writeBytes("[TABLE]\n");
		fouttab.writeBytes("table=" + tablename+"\n");
		fouttab.writeBytes("tablespace=PMO_DATA_64K\n");
		if (lobcolumns.toString().length() > 0)
		{
			//System.out.println("processTabDDL> LOB COLUMNS=" + lobcolumns);
			fouttab.writeBytes("lob_tbs=PMO_LOB_64K\n");
			lobcolumns.delete(0,STRINGBUFFER_SIZE-1);
		}
		tablename.delete(0,STRINGBUFFER_SIZE-1);		
	    }
	} //while true
	fintab.close();
    	return OK;
}
catch(Exception e)
{
    System.out.println("Exception in processTableDDL(: " + e);
    return ERROR;
}
}	//end of processTableDDL



//##############################################################################
//processIndexDDL:generate script for creating indexes
//##############################################################################
public static int processIndexDDL()
{
String line,line0;
StringBuffer linebuf,indexname,table;
RandomAccessFile finidx;
boolean searchidx, tbs_already_specified=false;
int inspos,pos,seppos,tabpos,linenum=0 ;
char let;

try
{
	finidx 		= new RandomAccessFile(in_idxscript,"r");
	indexname 	= new StringBuffer(STRINGBUFFER_SIZE);
	linebuf		= new StringBuffer(STRINGBUFFER_SIZE); 
	table		= new StringBuffer(STRINGBUFFER_SIZE); 
	searchidx	= false;

	while(true)
	{
	    if ((line0 = finidx.readLine()) ==null)
		break;
	    
	    //FORMAT THE LINE
	    line = line0.trim().toUpperCase();
	    linenum++;
	    if (line.length() < 1)
	    {
	    	continue;
	    }
	    
	    //CHECK CREATE INDEX & INDEX NAME
	    if (line.indexOf("CREATE")!= -1 && (pos=line.indexOf("INDEX")) != -1)
	    {
	    	//RESET EVERYTHING
	    	searchidx=true;
	    	indexname.delete(0,STRINGBUFFER_SIZE-1);
	    	table.delete(0,STRINGBUFFER_SIZE-1);
	    	pos = pos + 6;
	    	tbs_already_specified = false;
	   
	    }
	    else pos = 0;
	    if (searchidx==true)
	    {
	    	//INDEX NAME
	        for (int j=pos; j< line.length(); j++)
		{
		    	let = line.charAt(j);
		    	if ((let==' ' || let=='\t' || let=='(') && indexname.length() > 1)		    	
		    		break;
		    	else
		    		indexname.append(let);
		}
		
		tabpos = pos + indexname.length() + 3; //3=lenofON + 1
		//TABLE NAME
		for (int j=tabpos; j< line.length(); j++)
		{
		    	let = line.charAt(j);
		    	if ((let==' ' || let=='\t' || let=='(') && table.length() > 1)		    	
		    		break;
		    	else
		    		table.append(let);
		}
		//System.out.println("processIdxDDL>INDEX="+indexname+", tabpos="+tabpos+", pos="+pos+", TABLE=" + table + "\n");		
		searchidx=false;
	    }
	    
	    if (line.indexOf(";")!= -1)
	    {
	    	fouttab.writeBytes("\n[INDEX]\n");
	    	fouttab.writeBytes("index=" + indexname + "\n");
	    	fouttab.writeBytes("table=" + table +"\n"); 
	    	fouttab.writeBytes("tablespace=PMO_IDX_64K\n");
	    }
	    
	} //while true
	finidx.close();
	return OK;
}
catch(Exception e)
{
    System.out.println("Exception in processIndexDDL(: " + e);
    return ERROR;
}
}	//end of processIndexDDL


//***************************** END OF CLASS INSTALL ******************************
} //end of class genPreInstall
